@@ -1,3 +1,4 @@
+import sys
 import time
 import logging
 from threading import Thread
@@ -13,14 +14,15 @@ from .jobs import CFGGenerationJob
 from .object_container import ObjectContainer
 from .log import LogRecord, LogDumpHandler
 from ..logic import GlobalInfo
-from ..logic.threads import gui_thread_schedule_async
+from ..logic.threads import gui_thread_schedule_async, gui_thread_schedule
 from ..logic.debugger import DebuggerListManager, DebuggerManager
 from ..logic.debugger.simgr import SimulationDebugger
+from ..data.trace import Trace
+from ..data.breakpoint import BreakpointManager
 
 if TYPE_CHECKING:
     from ..ui.workspace import Workspace
 
-
 class Instance:
     """
     An object to give access to normal angr project objects like a Project, CFG, and other analyses.
@@ -33,7 +35,8 @@ class Instance:
     def __init__(self):
         # pylint:disable=import-outside-toplevel)
         # delayed import
-        from ..ui.views.interaction_view import PlainTextProtocol, BackslashTextProtocol, ProtocolInteractor, SavedInteraction
+        from ..ui.views.interaction_view import PlainTextProtocol, BackslashTextProtocol, ProtocolInteractor,\
+            SavedInteraction
 
         self._live = False
         self.workspace: Optional['Workspace'] = None
@@ -41,6 +44,7 @@ class Instance:
         self.jobs = []
         self._jobs_queue = Queue()
         self.current_job = None
+        self.worker_thread = None
 
         self.extra_containers = {}
         self._container_defaults = {}
@@ -63,7 +67,10 @@ class Instance:
                                 List[Type[ProtocolInteractor]],
                                 'Available interaction protocols')
         self.register_container('log', lambda: [], List[LogRecord], 'Saved log messages')
+        self.register_container('current_trace', lambda: None, Type[Trace], 'Currently selected trace')
+        self.register_container('traces', lambda: [], List[Trace], 'Global traces list')
 
+        self.breakpoint_mgr = BreakpointManager()
         self.debugger_list_mgr = DebuggerListManager()
         self.debugger_mgr = DebuggerManager(self.debugger_list_mgr)
 
@@ -253,26 +260,37 @@ class Instance:
         t = Thread(target=target, name=name, args=args if args else tuple())
         t.daemon = True
         t.start()
+        return t
 
     def _start_worker(self):
-        self._start_daemon_thread(self._worker, 'angr-management Worker Thread')
+        self.worker_thread = self._start_daemon_thread(self._worker, 'angr-management Worker Thread')
 
     def _worker(self):
+
         while True:
             if self._jobs_queue.empty():
-                gui_thread_schedule_async(self._set_status, args=("Ready.",))
+                gui_thread_schedule(GlobalInfo.main_window.progress_done, args=())
+
+            if self.workspace is not None and any(job.blocking for job in self.jobs):
+                gui_thread_schedule(self.workspace.main_window._progress_dialog.hide, args=())
 
             job = self._jobs_queue.get()
-            gui_thread_schedule_async(self._set_status, args=("Working...",))
+            gui_thread_schedule_async(GlobalInfo.main_window.progress, args=("Working...", 0.0))
+
+            if any(job.blocking for job in self.jobs):
+                if self.workspace.main_window.isVisible():
+                    gui_thread_schedule(self.workspace.main_window._progress_dialog.show, args=())
 
             try:
                 self.current_job = job
                 result = job.run(self)
                 self.current_job = None
-            except Exception as e: # pylint: disable=broad-except
+            except (Exception, KeyboardInterrupt) as e: # pylint: disable=broad-except
+                sys.last_traceback = e.__traceback__
                 self.current_job = None
                 self.workspace.log('Exception while running job "%s":' % job.name)
                 self.workspace.log(e)
+                self.workspace.log("Type %debug to debug it")
             else:
                 gui_thread_schedule_async(job.finish, args=(self, result))
 
@@ -306,6 +324,11 @@ class Instance:
             self.extra_containers[name].am_obj = self._container_defaults[name][0]()
             self.extra_containers[name].am_event(**kwargs)
 
+        for dbg in list(self.debugger_list_mgr.debugger_list):
+            self.debugger_list_mgr.remove_debugger(dbg)
+
+        self.breakpoint_mgr.clear()
+
     def _update_simgr_debuggers(self, **kwargs):  # pylint:disable=unused-argument
         sim_dbg = None
         for dbg in self.debugger_list_mgr.debugger_list:
