@@ -7,7 +7,7 @@ class CodeTaggingJob(Job):
     def __init__(self, on_finish=None):
         super(CodeTaggingJob, self).__init__(name="Code tagging", on_finish=on_finish)
 
-    def run(self, inst):
+    def _run(self, inst):
 
         func_count = len(inst.kb.functions)
         for i, func in enumerate(inst.kb.functions.values()):
