@@ -1,8 +1,7 @@
+from PySide2.QtCore import QObject, Signal
 
 from ...data.object_container import ObjectContainer
 
-from PySide2.QtCore import QObject, Signal
-
 
 class OperandHighlightMode:
     SAME_IDENT = 0
@@ -42,7 +41,6 @@ class InfoDock(QObject):
         self.hovered_block = ObjectContainer(None, 'The currently hovered block')
         self.hovered_edge = ObjectContainer(None, 'The currently hovered edge')
         self.selected_labels = ObjectContainer(set(), 'The currently selected labels')
-
         self.selected_qblock_code_obj = None
 
     @property
@@ -107,6 +105,8 @@ class InfoDock(QObject):
             self.selected_blocks.am_event()
 
     def select_instruction(self, insn_addr, unique=True, insn_pos=None, use_animation=True):
+        self.disasm_view.set_synchronized_cursor_address(insn_addr)
+
         self.unselect_all_labels()
         if insn_addr not in self.selected_insns:
             if unique:
@@ -158,6 +158,8 @@ class InfoDock(QObject):
             self.selected_operands.am_event()
 
     def select_label(self, label_addr):
+        self.disasm_view.set_synchronized_cursor_address(label_addr)
+
         # only one label can be selected at a time
         # also, clear selection of instructions and operands
         self.unselect_all_instructions()
