@@ -1,11 +1,13 @@
 # pylint:disable=no-self-use
 import os
 import logging
+import pickle
 import sys
 import time
+from functools import partial
 from typing import Optional, TYPE_CHECKING
 
-from PySide2.QtWidgets import QMainWindow, QTabWidget, QFileDialog, QProgressBar
+from PySide2.QtWidgets import QMainWindow, QTabWidget, QFileDialog, QProgressBar, QProgressDialog
 from PySide2.QtWidgets import QMessageBox, QShortcut, QTabBar
 from PySide2.QtGui import QResizeEvent, QIcon, QDesktopServices, QKeySequence
 from PySide2.QtCore import Qt, QSize, QEvent, QTimer, QUrl
@@ -18,7 +20,6 @@ try:
 except ImportError:
     AngrDB = None  # type: Optional[type]
 
-
 try:
     import archr
     import keystone
@@ -31,9 +32,9 @@ from ..daemon.client import ClientService
 from ..logic import GlobalInfo
 from ..data.instance import Instance
 from ..data.library_docs import LibraryDocs
-from ..data.jobs.loading import LoadTargetJob, LoadBinaryJob
+from ..data.jobs.loading import LoadTargetJob, LoadBinaryJob, LoadAngrDBJob
 from ..data.jobs import DependencyAnalysisJob
-from ..config import IMG_LOCATION, Conf
+from ..config import IMG_LOCATION, Conf, save_config
 from ..utils.io import isurl, download_url
 from ..utils.env import is_pyinstaller, app_root
 from ..errors import InvalidURLError, UnexpectedStatusCodeError
@@ -61,7 +62,8 @@ class MainWindow(QMainWindow):
     """
     The main window of angr management.
     """
-    def __init__(self, app: Optional['QApplication']=None, parent=None, show=True, use_daemon=False):
+
+    def __init__(self, app: Optional['QApplication'] = None, parent=None, show=True, use_daemon=False):
         super().__init__(parent)
 
         icon_location = os.path.join(IMG_LOCATION, 'angr.png')
@@ -79,15 +81,13 @@ class MainWindow(QMainWindow):
 
         self.toolbar_manager: ToolbarManager = ToolbarManager(self)
         self._progressbar = None  # type: QProgressBar
+        self._progress_dialog = None # type: QProgressDialog
         self._load_binary_dialog = None
 
-        self._status = ""
-        self._progress = None
-
         self.defaultWindowFlags = None
 
         # menus
-        self._file_menu = None
+        self._file_menu = None  # FileMenu
         self._analyze_menu = None
         self._view_menu = None
         self._help_menu = None
@@ -131,30 +131,11 @@ class MainWindow(QMainWindow):
     def caption(self, v):
         self.setWindowTitle(v)
 
-    @property
-    def status(self):
-        return self._status
-
-    @status.setter
-    def status(self, v):
-        self._status = v
-        self.statusBar().showMessage(v)
-
-    @property
-    def progress(self):
-        return self._progress
-
-    @progress.setter
-    def progress(self, v):
-        self._progress = v
-        self._progressbar.show()
-        self._progressbar.setValue(v)
-
     #
     # Dialogs
     #
 
-    def _open_mainfile_dialog(self):
+    def open_mainfile_dialog(self):
         # pylint: disable=assigning-non-slot
         # https://github.com/PyCQA/pylint/issues/3793
         file_path, _ = QFileDialog.getOpenFileName(self, "Open a binary", Conf.last_used_directory,
@@ -201,7 +182,6 @@ class MainWindow(QMainWindow):
     #
 
     def _init_statusbar(self):
-
         self._progressbar = QProgressBar()
 
         self._progressbar.setMinimum(0)
@@ -210,6 +190,21 @@ class MainWindow(QMainWindow):
 
         self.statusBar().addPermanentWidget(self._progressbar)
 
+        self._progress_dialog = QProgressDialog("Waiting...", "Cancel", 0, 100, self)
+        self._progress_dialog.setAutoClose(False)
+        self._progress_dialog.setWindowFlags(self._progress_dialog.windowFlags() & ~Qt.WindowContextHelpButtonHint)
+        self._progress_dialog.setModal(True)
+        self._progress_dialog.setMinimumDuration(2**31 - 1)
+        def on_cancel():
+            if self.workspace is None:
+                return
+            for job in self.workspace.instance.jobs:
+                if job.blocking:
+                    job.keyboard_interrupt()
+                    break
+        self._progress_dialog.canceled.connect(on_cancel)
+        self._progress_dialog.close()
+
     def _init_toolbars(self):
         for cls in (FileToolbar, DebugToolbar):
             self.toolbar_manager.show_toolbar_by_class(cls)
@@ -225,6 +220,9 @@ class MainWindow(QMainWindow):
         self._help_menu = HelpMenu(self)
         self._plugin_menu = PluginMenu(self)
 
+        for path in Conf.recent_files:
+            self._file_menu.add_recent(path)
+
         # TODO: Eventually fix menu bars to have native support on MacOS
         # if on a Mac, don't use the native menu bar (bug mitigation from QT)
         if sys.platform == 'darwin':
@@ -261,10 +259,12 @@ class MainWindow(QMainWindow):
                 self.caption = "Loaded from stream"
             else:
                 self.caption = os.path.basename(self.workspace.instance.project.filename)
+
         self.workspace.instance.project.am_subscribe(set_caption)
 
         self.tab = self.central_widget.findChild(QTabBar)
         self.tab.tabBarClicked.connect(self.on_center_tab_clicked)
+
     #
     # Shortcuts
     #
@@ -280,14 +280,23 @@ class MainWindow(QMainWindow):
         """
 
         center_dockable_views = self.workspace.view_manager.get_center_views()
-        for i in range(1, len(center_dockable_views)+1):
-            QShortcut(QKeySequence('Ctrl+'+str(i)), self, center_dockable_views[i-1].raise_)
+        for i in range(1, len(center_dockable_views) + 1):
+            QShortcut(QKeySequence('Ctrl+' + str(i)), self, center_dockable_views[i - 1].raise_)
 
         QShortcut(QKeySequence("Ctrl+I"), self, self.interrupt_current_job)
 
         # Raise the DisassemblyView after everything has initialized
         center_dockable_views[0].raise_()
 
+        # Toggle exec breakpoint
+        QShortcut(QKeySequence(Qt.Key_F2), self, self.workspace.toggle_exec_breakpoint)
+
+        # Single step
+        QShortcut(QKeySequence(Qt.Key_F7), self, self.workspace.step_forward)
+
+        # Run
+        QShortcut(QKeySequence(Qt.Key_F9), self, self.workspace.continue_forward)
+
     #
     # Plugins
     #
@@ -462,17 +471,27 @@ class MainWindow(QMainWindow):
         self.workspace.reload()
 
     def open_file_button(self):
-        file_path = self._open_mainfile_dialog()
+        file_path = self.open_mainfile_dialog()
         if not file_path:
             return
         self.load_file(file_path)
 
+    def open_trace_file_button(self):
+        file_path, _ = QFileDialog.getOpenFileName(self, "Open a trace file", Conf.last_used_directory,
+                                                   "All files (*);;"
+                                                   "Trace files (*.trace);;",
+                                                   )
+        Conf.last_used_directory = os.path.dirname(file_path)
+        if not file_path:
+            return
+        self.load_trace_file(file_path)
+
     def open_docker_button(self):
         required = {
-            'archr: git clone https://github.com/angr/archr && cd archr && pip install -e .':archr,
-            'keystone: pip install --no-binary keystone-engine keystone-engine':keystone
-            }
-        is_missing = [ key for key, value in required.items() if value is None ]
+            'archr: git clone https://github.com/angr/archr && cd archr && pip install -e .': archr,
+            'keystone: pip install --no-binary keystone-engine keystone-engine': keystone
+        }
+        is_missing = [key for key, value in required.items() if value is None]
         if len(is_missing) > 0:
             req_msg = 'You need to install the following:\n\n\t' + '\n\t'.join(is_missing)
             req_msg += '\n\nInstall them to enable this functionality.'
@@ -487,16 +506,51 @@ class MainWindow(QMainWindow):
         self.workspace.instance.add_job(LoadTargetJob(target))
         self.workspace.instance.img_name = img_name
 
+    def load_trace_file(self, file_path):
+        if isurl(file_path):
+            QMessageBox.critical(self,
+                                 "Unsupported Action",
+                                 "Downloading trace files is not yet supported."
+                                 "Please specify a path to a file on disk.")
+        else:
+            # File
+            if os.path.isfile(file_path):
+                try:
+                    with open(file_path, 'rb') as f:
+                        loaded_sim_state = pickle.load(f)
+                        analysis_params = {
+                            'end_state': loaded_sim_state,
+                            'start_addr': None,
+                            'end_addr': None,
+                            'block_addrs': None,
+                        }
+                        self.workspace.view_data_dependency_graph(analysis_params)
+                    self._recent_file(file_path)
+                except pickle.PickleError:
+                    QMessageBox.critical(self,
+                                         "Unable to load trace file",
+                                         "Trace file must contain a serialized SimState.")
+            else:
+                QMessageBox.critical(self,
+                                     "File not found",
+                                     f"angr management cannot open file {file_path}. "
+                                     "Please make sure that the file exists.")
+
     def load_file(self, file_path):
 
         if not isurl(file_path):
             # file
             if os.path.isfile(file_path):
+                if file_path.endswith(".trace"):
+                    self.workspace.load_trace_from_path(file_path)
+                    return
+
                 self.workspace.instance.binary_path = file_path
                 self.workspace.instance.original_binary_path = file_path
                 if file_path.endswith(".adb"):
                     self._load_database(file_path)
                 else:
+                    self._recent_file(file_path)
                     self.workspace.instance.add_job(LoadBinaryJob(file_path))
             else:
                 QMessageBox.critical(self,
@@ -574,6 +628,12 @@ class MainWindow(QMainWindow):
 
         return self._save_database(file_path)
 
+    def load_trace(self):
+        file_path, _ = QFileDialog.getOpenFileName(self, "Load trace", ".", "bintrace (*.trace)")
+        if not file_path:
+            return
+        self.workspace.load_trace_from_path(file_path)
+
     def preferences(self):
 
         # Open Preferences dialog
@@ -589,7 +649,7 @@ class MainWindow(QMainWindow):
     def run_induction_variable_analysis(self):
         self.workspace._get_or_create_disassembly_view().run_induction_variable_analysis()
 
-    def run_dependency_analysis(self, func_addr: Optional[int]=None, func_arg_idx: Optional[int]=None):
+    def run_dependency_analysis(self, func_addr: Optional[int] = None, func_arg_idx: Optional[int] = None):
         if self.workspace is None or self.workspace.instance is None:
             return
         dep_analysis_job = DependencyAnalysisJob(func_addr=func_addr, func_arg_idx=func_arg_idx)
@@ -610,9 +670,18 @@ class MainWindow(QMainWindow):
     # Other public methods
     #
 
+    def progress(self, status, progress):
+        self.statusBar().showMessage(f'Working... {status}')
+        self._progress_dialog.setLabelText(status)
+        self._progressbar.show()
+        self._progressbar.setValue(progress)
+        self._progress_dialog.setValue(progress)
+
+
     def progress_done(self):
-        self._progress = None
         self._progressbar.hide()
+        self.statusBar().showMessage("Ready.")
+        self._progress_dialog.hide()
 
     def bring_to_front(self):
         self.setWindowState((self.windowState() & ~Qt.WindowMinimized) | Qt.WindowActive)
@@ -623,42 +692,46 @@ class MainWindow(QMainWindow):
     # Private methods
     #
 
-    def _load_database(self, file_path):
+    def _recent_file(self, file_path):
+        file_path = os.path.abspath(file_path)
+        self._file_menu.add_recent(file_path)
+        Conf.recent_file(file_path)
+        save_config()
+
+    def _load_database(self, file_path: str):
 
         if AngrDB is None:
             QMessageBox.critical(None, 'Error',
                                  'AngrDB is not enabled. Maybe you do not have SQLAlchemy installed?')
             return
 
-        angrdb = AngrDB()
-        other_kbs = { }
-        extra_info = { }
-        try:
-            proj = angrdb.load(file_path, kb_names=["global", "pseudocode_variable_kb"], other_kbs=other_kbs,
-                               extra_info=extra_info)
-        except angr.errors.AngrIncompatibleDBError as ex:
-            QMessageBox.critical(None, 'Error',
-                                 "Failed to load the angr database because of compatibility issues.\n"
-                                 f"Details: {ex}")
-            return
-        except angr.errors.AngrDBError as ex:
-            QMessageBox.critical(None, 'Error',
-                                 'Failed to load the angr database.\n'
-                                 f'Details: {ex}')
-            _l.critical("Failed to load the angr database.", exc_info=True)
+
+        other_kbs = {}
+        extra_info = {}
+
+        job = LoadAngrDBJob(file_path, ["global", "pseudocode_variable_kb"], other_kbs=other_kbs, extra_info=extra_info)
+        job._on_finish = partial(self._on_load_database_finished, job)
+        self.workspace.instance.add_job(job)
+
+    def _on_load_database_finished(self, job: LoadAngrDBJob):
+        proj = job.project
+
+        if proj is None:
             return
 
+        self._recent_file(job.file_path)
+
         cfg = proj.kb.cfgs['CFGFast']
         cfb = proj.analyses.CFB()  # it will load functions from kb
 
-        self.workspace.instance.database_path = file_path
+        self.workspace.instance.database_path = job.file_path
 
         self.workspace.instance._reset_containers()
         self.workspace.instance.project = proj
         self.workspace.instance.cfg = cfg
         self.workspace.instance.cfb = cfb
-        if "pseudocode_variable_kb" in other_kbs:
-            self.workspace.instance.pseudocode_variable_kb = other_kbs["pseudocode_variable_kb"]
+        if "pseudocode_variable_kb" in job.other_kbs:
+            self.workspace.instance.pseudocode_variable_kb = job.other_kbs["pseudocode_variable_kb"]
         else:
             self.workspace.instance.initialize_pseudocode_variable_kb()
         self.workspace.instance.project.am_event(initialized=True)
@@ -666,7 +739,7 @@ class MainWindow(QMainWindow):
         # trigger callbacks
         self.workspace.reload()
         self.workspace.on_cfg_generated()
-        self.workspace.plugins.angrdb_load_entries(extra_info)
+        self.workspace.plugins.angrdb_load_entries(job.extra_info)
 
     def _save_database(self, file_path):
         if self.workspace.instance is None or self.workspace.instance.project.am_none:
@@ -682,9 +755,9 @@ class MainWindow(QMainWindow):
         angrdb = AngrDB(project=self.workspace.instance.project)
         extra_info = self.workspace.plugins.angrdb_store_entries()
         angrdb.dump(file_path, kbs=[
-                        self.workspace.instance.kb,
-                        self.workspace.instance.pseudocode_variable_kb,
-                    ],
+            self.workspace.instance.kb,
+            self.workspace.instance.pseudocode_variable_kb,
+        ],
                     extra_info=extra_info,
                     )
 
@@ -693,7 +766,7 @@ class MainWindow(QMainWindow):
 
     def _recalculate_view_sizes(self, old_size):
         adjustable_dockable_views = [dock for dock in self.workspace.view_manager.docks
-                                     if dock.widget().default_docking_position in ('left', 'bottom', )]
+                                     if dock.widget().default_docking_position in ('left', 'bottom',)]
 
         if not adjustable_dockable_views:
             return
