@@ -13,5 +13,10 @@ from .proximity_view import ProximityView
 from .types_view import TypesView
 from .hex_view import HexView
 from .log_view import LogView
+from .data_dep_view import DataDepView
 from .registers_view import RegistersView
 from .stack_view import StackView
+from .traces_view import TracesView
+from .trace_map_view import TraceMapView
+from .breakpoints_view import BreakpointsView
+from .call_explorer_view import CallExplorerView
