@@ -1,17 +1,27 @@
-from typing import TYPE_CHECKING, Callable, Optional, List
+import os
+from typing import TYPE_CHECKING, Callable, Optional, List, Union
 import logging
 import traceback
 
+from PySide2.QtWidgets import QMessageBox
 from angr.knowledge_plugins.functions.function import Function
 from angr import StateHierarchy
+from angr.misc.testing import is_testing
+from cle import SymbolType
 
 from ..logic.debugger import DebuggerWatcher
+from ..logic.debugger.bintrace import BintraceDebugger
+
 from ..config import Conf
+from ..data.breakpoint import Breakpoint, BreakpointType
+from ..data.trace import BintraceTrace, Trace
 from ..data.instance import ObjectContainer
+from ..data.jobs.loading import LoadBinaryJob
 from ..data.jobs import CodeTaggingJob, PrototypeFindingJob, VariableRecoveryJob, FlirtSignatureRecognitionJob
 from .views import (FunctionsView, DisassemblyView, SymexecView, StatesView, StringsView, ConsoleView, CodeView,
                     InteractionView, PatchesView, DependencyView, ProximityView, TypesView, HexView, LogView,
-                    RegistersView, StackView)
+                    DataDepView, RegistersView, StackView, TracesView, TraceMapView, BreakpointsView,
+                    CallExplorerView)
 from .view_manager import ViewManager
 from .menus.disasm_insn_context_menu import DisasmInsnContextMenu
 
@@ -37,6 +47,7 @@ class Workspace:
 
         self.view_manager: ViewManager = ViewManager(self)
         self.plugins: PluginManager = PluginManager(self)
+        self.variable_recovery_job: Optional[VariableRecoveryJob] = None
 
         self.current_screen = ObjectContainer(None, name="current_screen")
 
@@ -65,7 +76,7 @@ class Workspace:
             LogView(self, 'bottom'),
         ]
 
-        enabled_tabs =[x.strip() for x in Conf.enabled_tabs.split(",") if x.strip()]
+        enabled_tabs = [x.strip() for x in Conf.enabled_tabs.split(",") if x.strip()]
         for tab in self.default_tabs:
             if tab.__class__.__name__ in enabled_tabs or len(enabled_tabs)==0:
                 self.add_view(tab)
@@ -162,6 +173,11 @@ class Workspace:
             if view is not None:
                 view.reload()
 
+            # Clear the proximity view
+            view = self.view_manager.first_view_in_category('proximity')
+            if view is not None:
+                view.clear()
+
     def _on_flirt_signature_recognized(self):
         self.instance.add_job(
             PrototypeFindingJob(
@@ -170,20 +186,25 @@ class Workspace:
         )
 
     def _on_prototype_found(self):
-        self.instance.add_job(
-            VariableRecoveryJob(
-                on_finish=self.on_variable_recovered,
-                **self.instance.variable_recovery_args,
-            )
-        )
-
-    def on_variable_recovered(self):
         self.instance.add_job(
             CodeTaggingJob(
                 on_finish=self.on_function_tagged,
             )
         )
 
+        workers = 4 if not is_testing else 0  # disable multiprocessing on angr CI
+        self.variable_recovery_job = VariableRecoveryJob(
+            **self.instance.variable_recovery_args,
+            on_variable_recovered=self.on_variable_recovered,
+            workers=workers,
+        )
+        # prioritize the current function in display
+        disassembly_view = self.view_manager.first_view_in_category("disassembly")
+        if disassembly_view is not None:
+            if not disassembly_view.function.am_none:
+                self.variable_recovery_job.prioritize_function(disassembly_view.function.addr)
+        self.instance.add_job(self.variable_recovery_job)
+
     def on_function_tagged(self):
         # reload disassembly view
         if len(self.view_manager.views_by_category['disassembly']) == 1:
@@ -196,6 +217,16 @@ class Workspace:
             if view.current_function.am_obj is not None:
                 view.reload()
 
+    def on_variable_recovered(self, func_addr: int):
+        """
+        Called when variable information of the given function is available.
+
+        :param int func_addr:   Address of the function whose variable information is available.
+        """
+        disassembly_view = self.view_manager.first_view_in_category("disassembly")
+        if disassembly_view is not None:
+            disassembly_view.on_variable_recovered(func_addr)
+
     #
     # Public methods
     #
@@ -297,6 +328,56 @@ class Workspace:
         self.raise_view(view)
         view.setFocus()
 
+    def add_breakpoint(self, obj: Union[str, int], type_: Optional[str] = None, size: Optional[int] = None):
+        """
+        Convenience function to add a breakpoint.
+
+        Examples:
+        - `workspace.add_breakpoint(0x1234)` sets an execution breakpoint on address 0x1234
+        - `workspace.add_breakpoint('main')` sets an execution breakpoint on `main` function
+        - `workspace.add_breakpoint('global_value')` sets a write breakpoint on `global_value`
+        - `workspace.add_breakpoint('global_value', 'read', 1)` sets a 1-byte read breakpoint on `global_value`
+        """
+        if type(obj) is int:
+            addr = obj
+        elif type(obj) is str:
+            sym = self.instance.project.loader.find_symbol(obj)
+            if sym is None:
+                _l.error("Couldn't resolve '%s'", obj)
+                return
+            addr = sym.rebased_addr
+            if not size:
+                size = sym.size
+            if not type_:
+                if sym.type == SymbolType.TYPE_FUNCTION:
+                    type_ = 'execute'
+                else:
+                    type_ = 'write'
+        elif type(obj) is Function:
+            addr = obj.addr
+            if not type_:
+                type_ = 'execute'
+        else:
+            _l.error('Unexpected target object type. Expected int | str | Function')
+            return
+
+        if not size:
+            size = 1
+
+        bp_type_map = {
+            None: BreakpointType.Execute,
+            'execute': BreakpointType.Execute,
+            'write': BreakpointType.Write,
+            'read': BreakpointType.Read,
+        }
+        if type_ not in bp_type_map:
+            _l.error("Unknown breakpoint type '%s'. Expected %s",
+                     type_, ' | '.join(bp_type_map.keys()))
+            return
+
+        bp = Breakpoint(bp_type_map[type_], addr, size)
+        self.instance.breakpoint_mgr.add_breakpoint(bp)
+
     def set_comment(self, addr, comment_text):
         kb = self.instance.project.kb
         exists = addr in kb.comments
@@ -327,6 +408,10 @@ class Workspace:
             view = self._get_or_create_disassembly_view()
             view.decompile_current_function()
 
+    def view_data_dependency_graph(self, analysis_params: dict):
+        view = self._get_or_create_data_dependency_graph(analysis_params)
+        self.raise_view(view)
+
     def view_proximity_for_current_function(self, view=None):
         if view is None or view.category != "proximity":
             view = self._get_or_create_proximity_view()
@@ -370,6 +455,82 @@ class Workspace:
 
         self.raise_view(view)
 
+    def create_trace_debugger(self):
+        if self.instance.current_trace.am_none:
+            _l.error('No trace available')
+            return
+
+        if isinstance(self.instance.current_trace.am_obj, BintraceTrace):
+            dbg = BintraceDebugger(self.instance.current_trace.am_obj, self)
+            dbg.init()
+            self.instance.debugger_list_mgr.add_debugger(dbg)
+            self.instance.debugger_mgr.set_debugger(dbg)
+
+    def is_current_trace(self, trace: Optional[Trace]) -> bool:
+        return self.instance.current_trace.am_obj is trace
+
+    def set_current_trace(self, trace: Optional[Trace]):
+        self.instance.current_trace.am_obj = trace
+        self.instance.current_trace.am_event()
+
+    def remove_trace(self, trace: Trace):
+        if self.is_current_trace(trace):
+            self.set_current_trace(None)
+        self.instance.traces.remove(trace)
+        self.instance.traces.am_event(trace_removed=trace)
+        # Note: Debuggers and other objects may retain trace
+
+    def load_trace_from_path(self, path: str):
+        if not BintraceTrace.trace_backend_enabled():
+            QMessageBox.critical(None, "Error", "bintrace is not installed. Please install the bintrace package.")
+            return
+
+        _l.info("Opening trace %s", path)
+        trace = BintraceTrace.load_trace(path)
+
+        def on_complete():
+            if not self.instance.project.am_none:
+                self.instance.traces.append(trace)
+                self.instance.traces.am_event(trace_added=trace)
+                self.instance.current_trace.am_obj = trace
+                self.instance.current_trace.am_event()
+                self.show_traces_view()
+
+        if self.instance.project.am_none:
+            QMessageBox.information(None, "Creating project", "Creating new project from trace.")
+            self.create_project_from_trace(trace, on_complete)
+        else:
+            on_complete()
+
+    def create_project_from_trace(self, trace: Trace, on_complete: Callable):
+        thing = None
+        load_options = trace.get_project_load_options()
+
+        if load_options is None:
+            QMessageBox.warning(None, "Error", "Failed to determine load options for binary from trace. "
+                                      "Please select binary.")
+            load_options = {}
+        else:
+            thing = load_options["thing"]
+            load_options = load_options["load_options"]
+            if not os.path.exists(thing):
+                QMessageBox.warning(None, "Unable to find target binary!",
+                                          f"Unable to find the traced binary at: \n\n{thing}\n\n"
+                                          "Please select target binary.")
+                thing = None
+
+        if not thing:
+            thing = self.main_window.open_mainfile_dialog()
+
+        if not thing:
+            on_complete()
+            return
+
+        self.instance.binary_path = thing
+        self.instance.original_binary_path = thing
+        job = LoadBinaryJob(thing, load_options=load_options, on_finish=on_complete)
+        self.instance.add_job(job)
+
     def interact_program(self, img_name, view=None):
         if view is None or view.category != 'interaction':
             view = self._get_or_create_interaction_view()
@@ -379,7 +540,7 @@ class Workspace:
         view.setFocus()
 
     def log(self, msg):
-        if isinstance(msg, Exception):
+        if isinstance(msg, BaseException):
             msg = ''.join(traceback.format_exception(type(msg), msg, msg.__traceback__))
 
         console = self.view_manager.first_view_in_category('console')
@@ -475,6 +636,16 @@ class Workspace:
         self.raise_view(view)
         view.setFocus()
 
+    def show_traces_view(self):
+        view = self._get_or_create_traces_view()
+        self.raise_view(view)
+        view.setFocus()
+
+    def show_trace_map_view(self):
+        view = self._get_or_create_trace_map_view()
+        self.raise_view(view)
+        view.setFocus()
+
     def show_registers_view(self):
         view = self._get_or_create_registers_view()
         self.raise_view(view)
@@ -485,6 +656,16 @@ class Workspace:
         self.raise_view(view)
         view.setFocus()
 
+    def show_breakpoints_view(self):
+        view = self._get_or_create_breakpoints_view()
+        self.raise_view(view)
+        view.setFocus()
+
+    def show_call_explorer_view(self):
+        view = self._get_or_create_call_explorer_view()
+        self.raise_view(view)
+        view.setFocus()
+
     def show_console_view(self):
         view = self._get_or_create_console_view()
         self.raise_view(view)
@@ -495,6 +676,29 @@ class Workspace:
         self.raise_view(view)
         view.setFocus()
 
+    def toggle_exec_breakpoint(self):
+        if self.instance is None:
+            return
+
+        view: Optional[DisassemblyView] = self.view_manager.first_view_in_category('disassembly')
+        if view is not None:
+            selected_insns = view.current_graph.infodock.selected_insns
+            if selected_insns:
+                for insn in selected_insns:
+                    self.instance.breakpoint_mgr.toggle_exec_breakpoint(insn)
+
+    def step_forward(self, until_addr: Optional[int] = None):
+        if self.instance is None:
+            return
+
+        self.instance.debugger_mgr.debugger.step_forward(until_addr=until_addr)
+
+    def continue_forward(self):
+        if self.instance is None:
+            return
+
+        self.instance.debugger_mgr.debugger.continue_forward()
+
     #
     # Private methods
     #
@@ -597,6 +801,20 @@ class Workspace:
             self.add_view(view)
         return view
 
+    def _get_or_create_data_dependency_graph(self, analysis_params: dict) -> Optional[DataDepView]:
+        # Take the first data dependency view
+        view = self.view_manager.first_view_in_category('data_dependency')
+
+        if view is None:
+            # Create a new data dependency view
+            view = DataDepView(self, 'center')
+            self.add_view(view)
+
+        # Update DataDepView to utilize new analysis params
+        view.analysis_params = analysis_params
+
+        return view
+
     def _get_or_create_proximity_view(self) -> ProximityView:
         # Take the first proximity view
         view = self.view_manager.first_view_in_category("proximity")
@@ -661,6 +879,46 @@ class Workspace:
 
         return view
 
+    def _get_or_create_traces_view(self) -> TracesView:
+        # Take the first traces view
+        view = self.view_manager.first_view_in_category("traces")
+
+        if view is None:
+            view = TracesView(self, 'center')
+            self.add_view(view)
+
+        return view
+
+    def _get_or_create_trace_map_view(self) -> TraceMapView:
+        # Take the first tracemap view
+        view = self.view_manager.first_view_in_category("tracemap")
+
+        if view is None:
+            view = TraceMapView(self, 'top')
+            self.add_view(view)
+
+        return view
+
+    def _get_or_create_breakpoints_view(self) -> BreakpointsView:
+        # Take the first breakpoints view
+        view = self.view_manager.first_view_in_category("breakpoints")
+
+        if view is None:
+            view = BreakpointsView(self, 'center')
+            self.add_view(view)
+
+        return view
+
+    def _get_or_create_call_explorer_view(self) -> CallExplorerView:
+        # Take the first function call explorer view
+        view = self.view_manager.first_view_in_category('call_explorer')
+
+        if view is None:
+            view = CallExplorerView(self, 'right')
+            self.add_view(view)
+
+        return view
+
     #
     # UI-related Callback Setters & Manipulation
     #
